from django.conf import settings
from django.contrib.auth import get_user_model
from django.db.models.signals import post_save
from django.dispatch import receiver
from templated_email import send_templated_mail

User = get_user_model()


@receiver(post_save, sender=User)
def save_user_account(sender, instance, created, raw, **kwargs):
    if not raw and not created and instance.account:
        instance.account.save()


@receiver(post_save, sender=User)
def send_welcome_email(sender, instance, created, raw, **kwargs):
    if not raw and created and settings.LP_ACCOUNTS_WELCOME_EMAIL_ENABLED:
        send_templated_mail(
            template_name=settings.LP_ACCOUNTS_WELCOME_EMAIL_TEMPLATE,
            from_email=settings.LP_ACCOUNTS_WELCOME_EMAIL_SENDER,
            recipient_list=[instance.email],
            context={
                'username': instance.username,
                'full_name': instance.get_full_name(),
                'signup_date': instance.date_joined
            },
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-10-31 01:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('lp_accounts', '0002_user_is_confirmed'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='type',
            field=models.IntegerField(choices=[(1, 'Default'), (2, 'Google'), (3, 'Facebook'), (4, 'Github'), (5, 'LinkedIn')], default=1),
        ),
    ]



from django.conf import settings
from rest_framework import generics, permissions, status
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework.authtoken.models import Token
from rest_framework.response import Response
from templated_email import send_templated_mail
from .serializers import AccountSerializer, PasswordResetSerializer, PasswordUpdateSerializer, AuthTokenSerializer
from .models import User
from rest_condition import Or
from .permissions import IsAnonUser


class AccountCreate(generics.CreateAPIView):
    model = User
    serializer_class = AccountSerializer
    permission_classes = [
        Or(IsAnonUser, permissions.IsAdminUser)
    ]

    def perform_create(self, serializer):
        instance = serializer.save()
        Token.objects.get_or_create(user=instance)


class AccountDetails(generics.RetrieveAPIView):
    model = User
    serializer_class = AccountSerializer
    queryset = User.objects.all()
    permission_classes = [
        permissions.IsAuthenticated
    ]

    def get_object(self):
        return self.request.user


class AccountUpdate(generics.UpdateAPIView):
    model = User
    serializer_class = AccountSerializer
    queryset = User.objects.all()
    permission_classes = [
        permissions.IsAuthenticated
    ]

    def get_object(self):
        return self.request.user

    def update(self, request, *args, **kwargs):
        partial = kwargs.get('partial', False)
        instance = self.get_object()

        account_data = request.data.pop('account', None)
        if account_data:
            account = instance.account
            account_serializer_class = self.get_serializer().get_account_serializer(account_data).__class__
            account_serializer = account_serializer_class(account, data=account_data, partial=partial)
            account_serializer.is_valid(raise_exception=True)
            self.perform_update(account_serializer)

        return super(AccountUpdate, self).update(request, *args, **kwargs)


class PasswordReset(generics.GenericAPIView):
    serializer_class = PasswordResetSerializer
    permission_classes = {
        IsAnonUser
    }

    def post(self, request):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        token, created = Token.objects.get_or_create(user=serializer.user)
        send_templated_mail(
            template_name=settings.LP_ACCOUNTS_PASSWORD_RESET_TEMPLATE,
            from_email=settings.LP_ACCOUNTS_PASSWORD_RESET_SENDER,
            recipient_list=[request.data['email']],
            context={
                'username': serializer.user.username,
                'full_name': serializer.user.get_full_name(),
                'signup_date': serializer.user.date_joined,
                'token': token.key,
            },
        )

        return Response(status=status.HTTP_204_NO_CONTENT)


class PasswordUpdate(generics.GenericAPIView):
    model = User
    serializer_class = PasswordUpdateSerializer
    queryset = User.objects.all()
    permission_classes = [
        IsAnonUser
    ]

    def post(self, request):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        serializer.user.set_password(request.data['password'])
        serializer.user.save()

        return Response(status=status.HTTP_200_OK)


class ObtainAuthToken(ObtainAuthToken):
    serializer_class = AuthTokenSerializer
    permission_classes = [
        IsAnonUser
    ]



from django.db import models
from django.contrib.auth.models import AbstractUser
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType


# Create your models here.
class User(AbstractUser):
    TYPE_DEFAULT = 1
    TYPE_GOOGLE = 2
    TYPE_FACEBOOK = 3
    TYPE_GITHUB = 4
    TYPE_LINKEDIN = 5
    TYPES = (
        (TYPE_DEFAULT, 'Default'),
        (TYPE_GOOGLE, 'Google'),
        (TYPE_FACEBOOK, 'Facebook'),
        (TYPE_GITHUB, 'Github'),
        (TYPE_LINKEDIN, 'LinkedIn'),
    )

    type = models.IntegerField(choices=TYPES, default=TYPE_DEFAULT)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, null=True)
    account_id = models.PositiveIntegerField(null=True)
    account = GenericForeignKey('content_type', 'account_id')

    def __str__(self):
        return self.username


class BaseAccount(models.Model):
    class Meta:
        abstract = True



from collections import OrderedDict
from rest_framework import serializers


def serializer_factory(mdl):
    class Serializer(serializers.ModelSerializer):
        class Meta:
            model = mdl
            fields = '__all__'

    return Serializer



from django.conf import settings
import sys


default_app_config = 'lp_accounts.apps.AccountConfig'

settings.INSTALLED_APPS += [
    'generic_relations',
]

# Flag for Testing Environment
settings.TESTING = len(sys.argv) > 1 and sys.argv[1] == 'test'

# Extend Native User Model
settings.AUTH_USER_MODEL = 'lp_accounts.User'

# REST Framework Configuration
settings.REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.TokenAuthentication',
    )
}

# Configure Account Serializers
settings.LP_ACCOUNTS_SERIALIZERS = {}

# Configure Templated Email
settings.TEMPLATED_EMAIL_TEMPLATE_DIR = getattr(
    settings, 'TEMPLATED_EMAIL_TEMPLATE_DIR', 'templated_email/')
settings.TEMPLATED_EMAIL_FILE_EXTENSION = getattr(
    settings, 'TEMPLATED_EMAIL_FILE_EXTENSION', 'email'
)

# Welcome Email Configuration
settings.LP_ACCOUNTS_WELCOME_EMAIL_ENABLED = getattr(
    settings, 'LP_ACCOUNTS_WELCOME_EMAIL_ENABLED', False)
settings.LP_ACCOUNTS_WELCOME_EMAIL_TEMPLATE = getattr(
    settings, 'LP_ACCOUNTS_WELCOME_EMAIL_TEMPLATE', 'welcome')
settings.LP_ACCOUNTS_WELCOME_EMAIL_SENDER = getattr(
    settings, 'LP_ACCOUNTS_WELCOME_EMAIL_SENDER', 'support@launchpeer.com')

# Reset Password Email Configuration
settings.LP_ACCOUNTS_PASSWORD_RESET_TEMPLATE = getattr(
    settings, 'LP_ACCOUNTS_FORGOT_PASSWORD_TEMPLATE', 'passwordreset'
)
settings.LP_ACCOUNTS_PASSWORD_RESET_SENDER = getattr(
    settings, 'LP_ACCOUNTS_PASSWORD_RESET_SENDER', 'support@launchpeer.com'
)

# Google Sign-In Integration
# https://developers.google.com/identity/sign-in/web/backend-auth
settings.LP_ACCOUNTS_GOOGLE_APP_ID = getattr(
    settings, 'LP_ACCOUNTS_GOOGLE_APP_ID', None)
settings.LP_ACCOUNTS_GOOGLE_VALIDATION_URL = getattr(
    settings, 'LP_ACCOUNTS_GOOGLE_VALIDATION_URL', 'https://www.googleapis.com/oauth2/v3/tokeninfo?access_token=%s')

# Facebook Login Integration
# https://developers.facebook.com/docs/facebook-login/manually-build-a-login-flow#checktoken
settings.LP_ACCOUNTS_FACEBOOK_APP_ID = getattr(
    settings, 'LP_ACCOUNTS_FACEBOOK_APP_ID', None)
settings.LP_ACCOUNTS_FACEBOOK_CLIENT_SECRET = getattr(
    settings, 'LP_ACCOUNTS_FACEBOOK_CLIENT_SECRET', None)
settings.LP_ACCOUNTS_FACEBOOK_VALIDATION_URL = getattr(
    settings, 'FACEBOOK_VALIDATION_URL', 'https://graph.facebook.com/debug_token?input_token=%s&access_token=%s|%s')

# Github Login Integration
# Docs:
# https://developer.github.com/apps/building-integrations/setting-up-a-new-integration/about-integrations/
# Example:
# https://simpleisbetterthancomplex.com/tutorial/2016/10/24/how-to-add-social-login-to-django.html
settings.LP_ACCOUNTS_GITHUB_APP_ID = getattr(
    settings, 'LP_ACCOUNTS_GITHUB_APP_ID', None)
settings.LP_ACCOUNTS_GITHUB_CLIENT_SECRET = getattr(
    settings, 'LP_ACCOUNTS_GITHUB_CLIENT_SECRET', None)
settings.LP_ACCOUNTS_GITHUB_VALIDATION_URL = getattr(
    settings, 'LP_ACCOUNTS_GITHUB_VALIDATION_URL', 'https://api.github.com/applications/%s/tokens/%s')

# LinkedIn Login Integration
# Docs:
# https://developer.linkedin.com/docs/oauth2
settings.LP_ACCOUNTS_LINKEDIN_APP_ID = getattr(
    settings, 'LP_ACCOUNTS_LINKEDIN_APP_ID', None)
settings.LP_ACCOUNTS_LINKEDIN_APP_SECRET = getattr(
    settings, 'LP_ACCOUNTS_LINKEDIN_APP_SECRET', None)



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.core.urlresolvers import reverse
from .models import User


class UserAdmin(UserAdmin):
    list_display = ('username', 'email', 'type', 'account')

    def account(self, obj):
        account = obj.account
        if account:
            url = reverse('admin:%s_%s_change' % (account._meta.app_label, account._meta.model_name), args=[account.id])
            return u'<a href="%s">Edit %s</a>' % (url, account)

        return account

    account.allow_tags = True

UserAdmin.fieldsets += (
    ('Additional Data', {'fields': ('type',)}),
)
admin.site.register(User, UserAdmin)
